% This function computes the gradient of the measure of manipulability,
% used in modifying a trajector online to avoid a singularity. This is
% based on ``A real-time approach for singularity avoidance in
% Resolved Motion Rate Control of Robotic Manipulators.'' (except they do
% it symbolically, and I'm doing it numerically.

function grad_mom = compute_grad_mom(q, delta_q, jacobian_func, T_6_EE)
%#codegen

% The gradient has six elements, each the partial derivative with respect
% to a different joint variable. We will do this numerically.

q1_delta = [q(1) - delta_q; q(2:6)];
q2_delta = [q(1); q(2) - delta_q; q(3:6)];
q3_delta = [q(1:2); q(3) - delta_q; q(4:6)];
q4_delta = [q(1:3); q(4) - delta_q; q(5:6)];
q5_delta = [q(1:4); q(5) - delta_q; q(6)];
q6_delta = [q(1:5); q(6) - delta_q];

mom_q = compute_manipulability(q, jacobian_func, T_6_EE);

mom_dq1 = compute_manipulability(q1_delta, jacobian_func, T_6_EE);
mom_dq2 = compute_manipulability(q2_delta, jacobian_func, T_6_EE);
mom_dq3 = compute_manipulability(q3_delta, jacobian_func, T_6_EE);
mom_dq4 = compute_manipulability(q4_delta, jacobian_func, T_6_EE);
mom_dq5 = compute_manipulability(q5_delta, jacobian_func, T_6_EE);
mom_dq6 = compute_manipulability(q6_delta, jacobian_func, T_6_EE);

% Compute the gradient:
grad_mom = [mom_q - mom_dq1, mom_q - mom_dq2, mom_q - mom_dq3, ...
    mom_q - mom_dq4, mom_q - mom_dq5, mom_q - mom_dq6]/delta_q;