% This function computes the measure of manipulability at q. This is based
% on an approach in ``A real-time approach for singularity avoidance in
% Resolved Motion Rate Control of Robotic Manipulators.''

function mom = compute_manipulability(q, jacobian_func, T_6_EE)
%#codegen

% First get the Jacobian:
J = jacobian_func(q, T_6_EE);

% Now the measure:
det_JJt = det(J*J');

% In order to prevent imaginary values due to numerical error, we will
% ensure that the determinant is never negative (which it shouldn't be.)
if det_JJt < 0
    det_JJt = 0;
end

mom = sqrt(det_JJt);